#include <stdint.h>
#include <multiboot.h>
#include <cpu.h>
#include <memory.h>
#include <terminal.h>
#include <debug.h>
#include <proc.h>
#include <musl-glue.h>

void TEMP_test_scheduler();

struct kernel_boot_data_st kernel_boot_data;

void kmain(uint64_t multiboot_magic, void *multiboot_data)
{
    musl_init();
    debug_info("Started kernel\n");
    multiboot_init(multiboot_magic, P2V(multiboot_data));
    debug_info("Kernel loaded with command line: \"%s\" by <%s>\n",
        kernel_boot_data.commandline,
        kernel_boot_data.bootloader);

    cpu_init();
    debug_info("Set up boot CPU\n");
    memory_init();
    debug_info("Set up memory management\n");

    terminal_init();
    debug_info("Set up debug terminal\n");

    debug_info("Boot complete\n");


    TEMP_test_scheduler();

    debug("Read MSR APIC_BASE: %lx\n", read_msr(0x0000001B));
    debug("Read MSR SYSCALL_TARGET: %lx\n", read_msr(0xC0000082));
    write_msr(0xC0000082, 0x1234567890abcdef);
    debug("Read MSR SYSCALL_TARGET: %lx\n", read_msr(0xC0000082));

    start_scheduler();

    PANIC("End of kernel function!");
    
    debug_info("Broke out of panic");
    for(;;);
}